#include <bits/stdc++.h>

#define MAX_N 50005
#define endl '\n'

typedef unsigned long long ull;

struct edge
{
    int i, j;
};

std::vector<edge> graph = {edge {0, 0}};

auto find_connected(int n, int m, int a) -> int
{
    // Stores which nodes we reached/visited
    std::bitset<MAX_N> reached;

    // A connected graph with N nodes has at least N-1 edges
    // That means we need to have at least N-1 nodes on the interval (i, j)
    int min_b = n + a - 2;

    // BFS
    std::queue<int> nodes;
    nodes.push(1);

    while(!nodes.empty()) {
        auto node = nodes.front();
        nodes.pop();

        if(reached[node]) {
            continue;
        }

        reached[node] = 1;

        for(auto i = a; i <= min_b; ++i) {
            auto edge = graph[i];

            if(edge.i == node) {
                nodes.push(edge.j);
            } else if(edge.j == node) {
                nodes.push(edge.i);
            }
        }
    }

    // Check how much nodes we didn't reach
    auto needed = 0;
    for(auto i = 1; i <= n; ++i) {
        if(!reached[i]) {
            ++needed;
        }
    }

    // By adding new edges, try to eliminate all unreached nodes
    int curr_b;
    for(curr_b = min_b + 1; curr_b <= m && needed; ++curr_b) {
        auto edge = graph[curr_b];

        // If the newly found edge involves a node we have already reached
        // and allows us to reach a new node, include it in the range
        if(reached[edge.i] && !reached[edge.j]) {
            reached[edge.j] = 1;
            --needed;
        }
        else if(reached[edge.j] && !reached[edge.i]) {
            reached[edge.i] = 1;
            --needed;
        }
    }

    return needed == 0 ? curr_b - 1 : -1;
}

int main()
{
    std::ios::sync_with_stdio(false);
    std::cin.tie(nullptr);
    std::cout.tie(nullptr);

    int n, m;
    std::cin >> n >> m;

    int u, v;
    for(auto i = 1; i <= m; ++i) {
        std::cin >> u >> v;
        graph.emplace_back(edge {u, v});
    }

    auto answer = 0;
    for(auto i = 1; i <= m; ++i) {
        auto j = find_connected(n, m, i);
        if(j != -1) {
            answer += m - j + 1;
        }
    }

    std::cout << answer << endl;

    return 0;
}
