#pragma GCC optimize("unroll-loops,no-stack-protector,btr-bb-exclusive,fast-math,unroll-all-loops")
#pragma GCC optimize("O3")
#pragma GCC target("tune=native,sse,sse2,popcnt")

#include <iostream>
#include <cmath>
#include <cassert>
#include <vector>
#include <map>
#include <set>
#include <utility>
#include <tuple>
#include <algorithm>
#include <string>
#include <unordered_set>
#include <unordered_map>

using namespace std;

#define int long long

class DSU {
public:
    int n;
    vector<int> p;
    vector<int> rank;
    int comps;
    vector< pair<int, int> > changes;

    DSU(int sz) {
        n = sz;
        comps = n;
        p.resize(n);
        for(int i = 0; i < n; i++) {
            p[i] = i;
        }
        rank.resize(n);
    }

    int get(int v) {
        if(p[v] == v) {
            return v;
        }
        return get(p[v]);
    }

    void unite(int u, int v) {
        u = get(u);
        v = get(v);
        if(u != v) {
            if(rank[u] > rank[v]) {
                swap(u, v);
            }
            changes.emplace_back(u, p[u]);
            p[u] = v;
            if(rank[u] == rank[v]) {
                changes.emplace_back(-(v + 1), rank[v]);
                rank[v]++;
            }
            changes.emplace_back(-1, comps);
            comps--;
        }
    }

    int save() {
        return changes.size();
    }
    void restore(int pos) {
        while(changes.size() > pos) {
            int key = changes.back().first;
            int value = changes.back().second;
            if(key == -1) {
                comps = value;
            } else if(key < 0) {
                rank[-key - 1] = value;
            } else {
                p[key] = value;
            }
            changes.pop_back();
        }
    }
};

const int K = 450;

signed main() {
    int n, m;
    cin >> n >> m;

    vector< pair<int, int> > edges(m);
    for(int i = 0; i < m; i++) {
        int u, v;
        cin >> u >> v;
        u--; v--;
        edges[i] = make_pair(u, v);
    }

    int ans = 0;
    for(int block = K; block < m; block += K) {
        DSU dsu(n), dsu1(n);
        for(int j = block; j <= m; j++) {
            if(j % K == 0) {
                int state = dsu.save();
                for(int i = block - 1; i >= 0 && i >= block - K; i--) {
                    dsu.unite(edges[i].first, edges[i].second);
                    dsu1.unite(edges[i].first, edges[i].second);
                    if(dsu.comps == 1 && dsu1.comps > 1) {
                        int state = dsu1.save();
                        for(int jj = j - K; ; jj++) {
                            dsu1.unite(edges[jj].first, edges[jj].second);
                            if(dsu1.comps == 1) {
                                ans += m - jj;
                                //cout << "Ans for " << i + 1 << ": " << i + 1 << ".." << jj + 1 << endl;
                                break;
                            }
                        }
                        dsu1.restore(state);
                    }
                }
                dsu.restore(state);
                dsu1 = dsu;
            }
            if(j != m) {
                dsu.unite(edges[j].first, edges[j].second);
            }
        }

        for(int i = block - 1; i >= 0 && i >= block - K; i--) {
            dsu.unite(edges[i].first, edges[i].second);
            dsu1.unite(edges[i].first, edges[i].second);
            if(dsu.comps == 1 && dsu1.comps > 1) {
                int state = dsu1.save();
                for(int jj = m - m % K; ; jj++) {
                    dsu1.unite(edges[jj].first, edges[jj].second);
                    if(dsu1.comps == 1) {
                        ans += m - jj;
                        //cout << "Ans for " << i + 1 << ": " << i + 1 << ".." << jj + 1 << endl;
                        break;
                    }
                }
                dsu1.restore(state);
            }
        }
    }

    for(int block = 0; block < m; block += K) {
        for(int i = block; i < block + K && i < m; i++) {
            DSU dsu(n);
            for(int j = i; j < block + K && j < m; j++) {
                dsu.unite(edges[j].first, edges[j].second);
                if(dsu.comps == 1) {
                    ans += m - j;
                    //cout << "Ans for " << i + 1 << ": " << i + 1 << ".." << j + 1 << endl;
                    break;
                }
            }
        }
    }

    cout << ans;
    return 0;
}

// 4 4 1 2 2 4 1 3 1 4
